package com.up.ti;

import java.io.*;

/**
 * 
 * @author Ricky
 */
public abstract class TIVariableFile {

    private short[] data;
    private String comment;
    public FILETYPE type;

    /**
     * 
     */
    public TIVariableFile() {
	data = new short[0];
	setComment("");
    }

    /**

     @param data
     */
    public TIVariableFile(short[] data) {
	this.data = data;
	setComment("");
    }

    /**

     @param data
     @param comment
     */
    public TIVariableFile(short[] data, String comment) {
	this.data = data;
	this.comment = comment;
    }

    /**

     @param data
     */
    public void setData(short[] data) {
	this.data = data;
    }

    /**

     @return
     */
    public short[] getData() {
	return data;
    }

    /**

     @param comment
     */
    public void setComment(String comment) {
	if (comment.length() < 42) {
	    comment = Util.padRight(comment, 42, (char)0);
	} else if (comment.length() > 42) {
	    comment = comment.substring(0, 41);
	}
	this.comment = comment;
    }

    /**

     @return
     */
    public String getComment() {
	return comment;
    }
    
    /**

     @param size
     */
    public void setSize(int size) {
	data = (short[])Util.resizeArray(data, size);
    }
    
    /**

     @return
     */
    public int getSize() {
	return data.length;
    }

    /**

     @param index
     @param value
     @return
     */
    public boolean setByte(int index, short value) {
	if (index < data.length) {
	    data[index] = value;
	    return true;
	}
	return false;
    }

    /**

     @param index
     @param value
     @return
     */
    public boolean setByte(int index, int value) {
	if (index < data.length) {
	    data[index] = (short)value;
	    return true;
	}
	return false;
    }

    /**

     @param data
     @return
     */
    public short calculateChecksum(short[] data) {
	short total = 0;
	for (int x = 0; x < data.length; x++) {
	    total += data[x];
	}
	return (short)(total % 65536);
    }
    
    /**
     * Writes the byte array to the file specified by <tt>name</tt>.
     * @param name Name of the file to write
     */
    protected void writeFile(String name) {
	FileOutputStream fos = null;
	try {
	    File file = new File(name);
	    file.createNewFile();
	    fos = new FileOutputStream(file);
	    for (int x = 0; x < 8; x++) {
		fos.write("**TI83F*".charAt(x));
	    }
	    fos.write(26);
	    fos.write(10);
	    fos.write(0);
	    for (int x = 0; x < 42; x++) {
		fos.write(comment.charAt(x));
	    }
	    fos.write(data.length % 256);
	    fos.write(data.length / 256);
	    for (int x = 0; x < data.length; x++) {
		fos.write(data[x]);
	    }
	    fos.write(calculateChecksum(data) % 256);
	    fos.write(calculateChecksum(data) / 256);
	} catch (IOException e) {
	    e.printStackTrace();
	} finally {
	    try {fos.close();} catch (Exception ex) {}
	}
    }

    /**
     * 
     * @param name
     */
    protected void readFile(String name) throws FileNotFoundException {
        File file = new File(name);
        try {
	    short[] tmpdata = getBytesFromFile(file);
	    comment = "";
	    for (int x = 0; x < 42; x++) {
		comment += Character.toString((char)tmpdata[11 + x]);
	    }
            data = new short[tmpdata[53] + tmpdata[54] * 256];
	    System.arraycopy(tmpdata, 55, data, 0, data.length);
        } catch (IOException ex) {ex.printStackTrace();}
    }

    /**

     @param name
     */
    protected void readFile(InputStream file) {
        try {
	    short[] tmpdata = getBytesFromFile(file);
	    comment = "";
	    for (int x = 0; x < 42; x++) {
		comment += Character.toString((char)tmpdata[11 + x]);
	    }
            data = new short[tmpdata[53] + tmpdata[54] * 256];
	    System.arraycopy(tmpdata, 55, data, 0, data.length);
        } catch (Exception ex) {
	    ex.printStackTrace();
	}
    }

    public abstract void saveFile(String name);

    public static short[] getBytesFromFile(File file) throws IOException {
        InputStream is = new FileInputStream(file);
        long length = file.length();
        if (length > Integer.MAX_VALUE) {
	    throw new IOException("File \"" + file.getName() + "\" is too large.");
        }
        short[] bytes = new short[(int)length];
	int pos = 0;
        while (pos < bytes.length && (bytes[pos] = (short)is.read()) >= 0) {
            pos++;
        }
        if (pos < bytes.length) {
            throw new IOException("Could not completely read file \"" + file.getName() + "\".");
        }
        is.close();
        return bytes;
    }

    public static short[] getBytesFromFile(InputStream file) throws IOException {
        InputStream is = file;
        short[] bytes = new short[100000];
	int pos = 0;
        while (pos < bytes.length && (bytes[pos] = (short)is.read()) >= 0) {
            pos++;
        }
        if (pos < bytes.length) {
            throw new IOException("Could not completely read file.");
        }
        is.close();
        return (short[])Util.resizeArray(bytes, pos);
    }
}
