package com.up.ti.data;

/**
 * Represents a single <tt>real</tt> value.
 * @author Ricky
 */
public class Real {
    
    public short exponent;
    public long mantissa;
    public byte sign;
    
    /**
     * Constructs a new Real based on the <tt>real</tt> format.
     * @param sign either -1 or 1
     * @param mantissa between 0 and 99,999,999,999,999
     * @param exponent between -99 and 99, with an offset of +128
     */
    public Real(int sign, long mantissa, int exponent) {
        if (exponent < 128 - 99 || exponent > 128 + 99) throw new NumberFormatException("Exponent out of bounds");
        this.exponent = (short)exponent;
        if (mantissa < 0 || mantissa > 99999999999999l) throw new NumberFormatException("Mantissa out of bounds");
        this.mantissa = mantissa;
        this.sign = (byte)sign;
    }
    
    public double getDouble() {
        return sign * mantissa * Math.pow(10, (double)(exponent - 13 - 128));
    }
}
