package com.up.ti.data;

import com.up.ti.SYNTAXTYPE;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

/**
 * Maintains a single complete set of Tokens, initialized with the default Ascii Token list of the TI 83/84+ calculators.
 * @author Ricky
 */
public class TokenSet {
    
    public Token[] tokenlist = getBasicTokenList();
    
    public short[] stringToData(String tokens) {
	Stack<Short> data = new Stack<Short>();
	int length = 0;
	while (tokens.length() > 0) {
	    int len = tokens.length() > 22 ? 22 : tokens.length();
	    while (tokenMatch(tokens.substring(0, len)) == null) {
		len--;
		if (len == 0) {
		    len = -1;
		    break;
		}
	    }
	    length++;
	    if (len == -1) {
		data.push((short)0xAF);
		tokens = tokens.substring(1, tokens.length());
	    } else {
		Token token = tokenMatch(tokens.substring(0, len));
		if (!token.istwobytes) {
		    data.push((short)token.byte1);
		    tokens = tokens.substring(len, tokens.length());
		} else {
		    length++;
		    data.push((short)token.byte1);
		    data.push((short)token.byte2);
		    tokens = tokens.substring(len, tokens.length());
		}
	    }
	}
        short[] ret = new short[data.size()];
        Short[] datas = data.toArray(new Short[0]);
        for (int x = 0; x < datas.length; x++) ret[x] = datas[x];
	return ret;
    }

    public String dataToString(short[] data) {
	String tokens = "";
	for (int x = 0; x < data.length; x++) {
	    for (int y = 0; y < tokenlist.length; y++) {
		if (tokenlist[y].byte1 == data[x]) {
		    if (tokenlist[y].istwobytes) {
			if (tokenlist[y].byte2 == data[x + 1]) {
			    tokens += tokenlist[y].token;
			    x++;
			    break;
			}
		    } else {
			tokens += tokenlist[y].token;
			break;
		    }
		}
		if (y == tokenlist.length - 1) {
		    tokens += "?";
		}
	    }
	}
	return tokens;
    }

    public Token findTokenFromData(short data, short data2) {
	for (int y = 0; y < tokenlist.length; y++) {
	    if (tokenlist[y].byte1 == data) {
		if (tokenlist[y].byte2 > -1) {
		    if (tokenlist[y].byte2 == data2) {
			return tokenlist[y];
		    }
		} else {
		    return tokenlist[y];
		}
	    }
	}
	return null;
    }

    private Token tokenMatch(String token) {
	for (int x = 0; x < tokenlist.length; x++) {
	    if (token.equals(tokenlist[x].token)) {
		return tokenlist[x];
	    }
	}
	return null;
    }

    public List<Token> tokenStartsWith(String token) {
        Stack<Token> s = new Stack<Token>();
        boolean allfound = false;
        int prev = 0;
        while (!allfound) {
            allfound = true;
            for (int x = prev; x < tokenlist.length; x++) {
                if (tokenlist[x].token.startsWith(token)) {
                    s.push(tokenlist[x]);
                    allfound = false;
                    prev = x + 1;
                    break;
                }
            }
        }
	return s;
    }
    
    public void changeTokens(String resc) {
        try {
            changeTokens(getClass().getResourceAsStream(resc));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void changeTokens(File file) {
        try {
            changeTokens(new FileInputStream(file));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public void changeTokens(InputStream file) {
        tokenlist = getBasicTokenList();
        Token[] basictokens = getBasicTokenList();
        boolean isAxe;
        try {
            BufferedReader br = new BufferedReader(new InputStreamReader(file));
            String line;
            while ((line = br.readLine()) != null) {
                if (line.startsWith("AXE")) {
                    isAxe = true;
                }
                if (line.startsWith("0:")) {
                    line = line.substring(2);
                    for (int y = 0; y < tokenlist.length; y++) {
                        if (tokenlist[y].byte1 == Integer.parseInt(line.split("\\|")[0])) {
                            if (tokenlist[y].istwobytes) {
                                if (tokenlist[y].byte2 == Integer.parseInt(line.split("\\|")[1])) {
                                    tokenlist[y].token = line.split("\\|")[2];
                                }
                            } else {
                                tokenlist[y].token = line.split("\\|")[2];
                            }
                        }
                    }
                }
                if (line.startsWith("1:")) {
                    line = line.substring(2);
                    for (int x = 0; x < tokenlist.length; x++) {
                        if (line.split("\\|")[0].equals(basictokens[x].token)) {
                            tokenlist[x].token = line.split("\\|")[1];
                        }
                    }
                }
            }
        } catch (Exception ex) {
            
        }
    }
    
    private final static String ii = "" + Character.toChars(0x0001D456)[0] + Character.toChars(0x0001D456)[1];
    private final static String in = "" + Character.toChars(0x0001D45B)[0] + Character.toChars(0x0001D45B)[1];
    public static Token[] getBasicTokenList() {
        return new Token[] {
            new Token(0x1, -1, "►DMS", SYNTAXTYPE.function),
            new Token(0x2, -1, "►Dec", SYNTAXTYPE.function),
            new Token(0x3, -1, "►Frac", SYNTAXTYPE.function),
            new Token(0x4, -1, "→", SYNTAXTYPE.operator),
            new Token(0x5, -1, "BoxPlot", SYNTAXTYPE.setting),
            new Token(0x6, -1, "[", SYNTAXTYPE.operator),
            new Token(0x7, -1, "]", SYNTAXTYPE.operator),
            new Token(0x8, -1, "{", SYNTAXTYPE.operator),
            new Token(0x9, -1, "}", SYNTAXTYPE.operator),
            new Token(0xA, -1, "ʳ", SYNTAXTYPE.operator),
            new Token(0xB, -1, "°", SYNTAXTYPE.operator),
            new Token(0xC, -1, "⁻¹", SYNTAXTYPE.operator),
            new Token(0xD, -1, "²", SYNTAXTYPE.operator),
            new Token(0xE, -1, "ᵀ", SYNTAXTYPE.operator), 
            new Token(0xF, -1, "³", SYNTAXTYPE.operator),
            new Token(0x10, -1, "(", SYNTAXTYPE.operator),
            new Token(0x11, -1, ")", SYNTAXTYPE.operator),
            new Token(0x12, -1, "round(", SYNTAXTYPE.function),
            new Token(0x13, -1, "pxl-Test(", SYNTAXTYPE.function),
            new Token(0x14, -1, "augment(", SYNTAXTYPE.function),
            new Token(0x15, -1, "rowSwap", SYNTAXTYPE.function),
            new Token(0x16, -1, "row+(", SYNTAXTYPE.function),
            new Token(0x17, -1, "*row(", SYNTAXTYPE.function),
            new Token(0x18, -1, "*row+(", SYNTAXTYPE.function),
            new Token(0x19, -1, "max(", SYNTAXTYPE.function),
            new Token(0x1A, -1, "min(", SYNTAXTYPE.function),
            new Token(0x1B, -1, "R►Pr(", SYNTAXTYPE.function),
            new Token(0x1C, -1, "R►Pθ(", SYNTAXTYPE.function),
            new Token(0x1D, -1, "P►Rx(", SYNTAXTYPE.function),
            new Token(0x1E, -1, "P►Ry(", SYNTAXTYPE.function),
            new Token(0x1F, -1, "median(", SYNTAXTYPE.function),
            new Token(0x20, -1, "randM(", SYNTAXTYPE.function),
            new Token(0x21, -1, "mean(", SYNTAXTYPE.function),
            new Token(0x22, -1, "solve(", SYNTAXTYPE.function),
            new Token(0x23, -1, "seq(", SYNTAXTYPE.function),
            new Token(0x24, -1, "fnInt(", SYNTAXTYPE.function),
            new Token(0x25, -1, "nDeriv(", SYNTAXTYPE.function),
            new Token(0x27, -1, "fMin(", SYNTAXTYPE.function),
            new Token(0x28, -1, "fMax(", SYNTAXTYPE.function),
            new Token(0x29, -1, " ", SYNTAXTYPE.operator),
            new Token(0x2A, -1, "\"", SYNTAXTYPE.operator),
            new Token(0x2B, -1, ",", SYNTAXTYPE.operator),
            new Token(0x2C, -1, ii, SYNTAXTYPE.operator),
            new Token(0x2D, -1, "!", SYNTAXTYPE.operator),
            new Token(0x2E, -1, "CubicReg ", SYNTAXTYPE.function),
            new Token(0x2F, -1, "QuartReg ", SYNTAXTYPE.function),
            new Token(0x30, -1, "0" , SYNTAXTYPE.number),
            new Token(0x31, -1, "1", SYNTAXTYPE.number),
            new Token(0x32, -1, "2", SYNTAXTYPE.number),
            new Token(0x33, -1, "3", SYNTAXTYPE.number),
            new Token(0x34, -1, "4", SYNTAXTYPE.number),
            new Token(0x35, -1, "5", SYNTAXTYPE.number),
            new Token(0x36, -1, "6", SYNTAXTYPE.number),
            new Token(0x37, -1, "7", SYNTAXTYPE.number),
            new Token(0x38, -1, "8", SYNTAXTYPE.number),
            new Token(0x39, -1, "9", SYNTAXTYPE.number),
            new Token(0x3A, -1, ".", SYNTAXTYPE.number),
            new Token(0x3B, -1, "ᴇ", SYNTAXTYPE.operator),
            new Token(0x3C, -1, " or ", SYNTAXTYPE.operator),
            new Token(0x3D, -1, " xor ", SYNTAXTYPE.operator),
            new Token(0x3E, -1, ":", SYNTAXTYPE.operator),
            new Token(0x3F, -1, "\n", SYNTAXTYPE.operator),
            new Token(0x40, -1, " and ", SYNTAXTYPE.operator),
            new Token(0x41, -1, "A", SYNTAXTYPE.variable),
            new Token(0x42, -1, "B", SYNTAXTYPE.variable),
            new Token(0x43, -1, "C", SYNTAXTYPE.variable),
            new Token(0x44, -1, "D", SYNTAXTYPE.variable),
            new Token(0x45, -1, "E", SYNTAXTYPE.variable),
            new Token(0x46, -1, "F", SYNTAXTYPE.variable),
            new Token(0x47, -1, "G", SYNTAXTYPE.variable),
            new Token(0x48, -1, "H", SYNTAXTYPE.variable),
            new Token(0x49, -1, "I", SYNTAXTYPE.variable),
            new Token(0x4A, -1, "J", SYNTAXTYPE.variable),
            new Token(0x4B, -1, "K", SYNTAXTYPE.variable),
            new Token(0x4C, -1, "L", SYNTAXTYPE.variable),
            new Token(0x4D, -1, "M", SYNTAXTYPE.variable),
            new Token(0x4E, -1, "N", SYNTAXTYPE.variable),
            new Token(0x4F, -1, "O", SYNTAXTYPE.variable),
            new Token(0x50, -1, "P", SYNTAXTYPE.variable),
            new Token(0x51, -1, "Q", SYNTAXTYPE.variable),
            new Token(0x52, -1, "R", SYNTAXTYPE.variable),
            new Token(0x53, -1, "S", SYNTAXTYPE.variable),
            new Token(0x54, -1, "T", SYNTAXTYPE.variable),
            new Token(0x55, -1, "U", SYNTAXTYPE.variable),
            new Token(0x56, -1, "V", SYNTAXTYPE.variable),
            new Token(0x57, -1, "W", SYNTAXTYPE.variable),
            new Token(0x58, -1, "X", SYNTAXTYPE.variable),
            new Token(0x59, -1, "Y", SYNTAXTYPE.variable),
            new Token(0x5A, -1, "Z", SYNTAXTYPE.variable),
            new Token(0x5B, -1, "θ", SYNTAXTYPE.variable),
            new Token(0x5C, 0x0, "[A]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x1, "[B]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x2, "[C]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x3, "[D]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x4, "[E]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x5, "[F]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x6, "[G]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x7, "[H]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x8, "[I]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x9, "[J]", SYNTAXTYPE.variable),
            new Token(0x5C, 0xB, "[L]", SYNTAXTYPE.variable),
            new Token(0x5C, 0xC, "[M]", SYNTAXTYPE.variable),
            new Token(0x5C, 0xD, "[N]", SYNTAXTYPE.variable),
            new Token(0x5C, 0xE, "[O]", SYNTAXTYPE.variable),
            new Token(0x5C, 0xF, "[P]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x10, "[Q]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x11, "[R]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x12, "[S]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x13, "[T]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x14, "[U]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x15, "[V]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x16, "[W]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x17, "[X]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x18, "[Y]", SYNTAXTYPE.variable),
            new Token(0x5C, 0x19, "[Z]", SYNTAXTYPE.variable),
            new Token(0x5D, 0x0, "L₁", SYNTAXTYPE.variable),
            new Token(0x5D, 0x1, "L₂", SYNTAXTYPE.variable),
            new Token(0x5D, 0x2, "L₃", SYNTAXTYPE.variable),
            new Token(0x5D, 0x3, "L₄", SYNTAXTYPE.variable),
            new Token(0x5D, 0x4, "L₅", SYNTAXTYPE.variable),
            new Token(0x5D, 0x5, "L₆", SYNTAXTYPE.variable),
            new Token(0x5D, 0x6, "L₇", SYNTAXTYPE.variable),
            new Token(0x5D, 0x7, "L₈", SYNTAXTYPE.variable),
            new Token(0x5D, 0x8, "L₉", SYNTAXTYPE.variable),
            new Token(0x5D, 0x9, "L₁₀", SYNTAXTYPE.variable),
            new Token(0x5D, 0xA, "L₁₁", SYNTAXTYPE.variable),
            new Token(0x5D, 0xB, "L₁₂", SYNTAXTYPE.variable),
            new Token(0x5D, 0xC, "L₁₃", SYNTAXTYPE.variable),
            new Token(0x5D, 0xD, "L₁₄", SYNTAXTYPE.variable),
            new Token(0x5D, 0xE, "L₁₅", SYNTAXTYPE.variable),
            new Token(0x5D, 0xF, "L₁₆", SYNTAXTYPE.variable),
            new Token(0x5D, 0xF, "L₁₆", SYNTAXTYPE.variable),
            new Token(0x5D, 0x10, "L₁₇", SYNTAXTYPE.variable),
            new Token(0x5D, 0x11, "L₁₈", SYNTAXTYPE.variable),
            new Token(0x5D, 0x12, "L₁₉", SYNTAXTYPE.variable),
            new Token(0x5D, 0x13, "L₂₀", SYNTAXTYPE.variable),
            new Token(0x5E, 0x10, "Y₁", SYNTAXTYPE.variable),
            new Token(0x5E, 0x11, "Y₂", SYNTAXTYPE.variable),
            new Token(0x5E, 0x12, "Y₃", SYNTAXTYPE.variable),
            new Token(0x5E, 0x13, "Y₄", SYNTAXTYPE.variable),
            new Token(0x5E, 0x14, "Y₅", SYNTAXTYPE.variable),
            new Token(0x5E, 0x15, "Y₆", SYNTAXTYPE.variable),
            new Token(0x5E, 0x16, "Y₇", SYNTAXTYPE.variable),
            new Token(0x5E, 0x17, "Y₈", SYNTAXTYPE.variable),
            new Token(0x5E, 0x18, "Y₉", SYNTAXTYPE.variable),
            new Token(0x5E, 0x19, "Y₀", SYNTAXTYPE.variable),
            new Token(0x5E, 0x20, "X₁⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x21, "Y₁⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x22, "X₂⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x23, "Y₂⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x24, "X₃⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x25, "Y₃⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x26, "X₄⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x27, "Y₄⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x28, "Y₅⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x29, "Y₅⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x2A, "X₆⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x2B, "Y₆⊤", SYNTAXTYPE.variable),
            new Token(0x5E, 0x40, "r₁", SYNTAXTYPE.variable),
            new Token(0x5E, 0x41, "r₂", SYNTAXTYPE.variable),
            new Token(0x5E, 0x42, "r₃", SYNTAXTYPE.variable),
            new Token(0x5E, 0x43, "r₄", SYNTAXTYPE.variable),
            new Token(0x5E, 0x44, "r₅", SYNTAXTYPE.variable),
            new Token(0x5E, 0x45, "r₆", SYNTAXTYPE.variable),
            new Token(0x5E, 0x80, "ˢu", SYNTAXTYPE.nothing),
            new Token(0x5E, 0x81, "ˢv", SYNTAXTYPE.nothing),
            new Token(0x5F, -1, "prgm", SYNTAXTYPE.keyword),
            new Token(0x60, 0x0, "Pic1", SYNTAXTYPE.variable),
            new Token(0x60, 0x1, "Pic2", SYNTAXTYPE.variable),
            new Token(0x60, 0x2, "Pic3", SYNTAXTYPE.variable),
            new Token(0x60, 0x3, "Pic4", SYNTAXTYPE.variable),
            new Token(0x60, 0x4, "Pic5", SYNTAXTYPE.variable),
            new Token(0x60, 0x5, "Pic6", SYNTAXTYPE.variable),
            new Token(0x60, 0x6, "Pic7", SYNTAXTYPE.variable),
            new Token(0x60, 0x7, "Pic8", SYNTAXTYPE.variable),
            new Token(0x60, 0x8, "Pic9", SYNTAXTYPE.variable),
            new Token(0x60, 0x9, "Pic10", SYNTAXTYPE.variable),
            new Token(0x60, 0xA, "Pic11", SYNTAXTYPE.variable),
            new Token(0x60, 0xB, "Pic12", SYNTAXTYPE.variable),
            new Token(0x60, 0xC, "Pic13", SYNTAXTYPE.variable),
            new Token(0x60, 0xD, "Pic14", SYNTAXTYPE.variable),
            new Token(0x60, 0xE, "Pic15", SYNTAXTYPE.variable),
            new Token(0x60, 0xF, "Pic16", SYNTAXTYPE.variable),
            new Token(0x60, 0x10, "Pic17", SYNTAXTYPE.variable),
            new Token(0x60, 0x11, "Pic18", SYNTAXTYPE.variable),
            new Token(0x60, 0x12, "Pic19", SYNTAXTYPE.variable),
            new Token(0x60, 0x13, "Pic20", SYNTAXTYPE.variable),
            new Token(0x61, 0x0, "GDB1", SYNTAXTYPE.variable),
            new Token(0x61, 0x1, "GDB2", SYNTAXTYPE.variable),
            new Token(0x61, 0x2, "GDB3", SYNTAXTYPE.variable),
            new Token(0x61, 0x3, "GDB4", SYNTAXTYPE.variable),
            new Token(0x61, 0x4, "GDB5", SYNTAXTYPE.variable),
            new Token(0x61, 0x5, "GDB6", SYNTAXTYPE.variable),
            new Token(0x61, 0x6, "GDB7", SYNTAXTYPE.variable),
            new Token(0x61, 0x7, "GDB8", SYNTAXTYPE.variable),
            new Token(0x61, 0x8, "GDB9", SYNTAXTYPE.variable),
            new Token(0x61, 0x9, "GDB0", SYNTAXTYPE.variable),
            new Token(0x62, 0x1, "RegEq", SYNTAXTYPE.nothing),
            new Token(0x62, 0x2, "ˢn", SYNTAXTYPE.variable),
            new Token(0x62, 0x3, "x̄", SYNTAXTYPE.variable),
            new Token(0x62, 0x4, "Σx", SYNTAXTYPE.variable),
            new Token(0x62, 0x5, "Σx²", SYNTAXTYPE.variable),
            new Token(0x62, 0x6, "Sx", SYNTAXTYPE.variable),
            new Token(0x62, 0x7, "σx", SYNTAXTYPE.variable),
            new Token(0x62, 0x8, "MinX", SYNTAXTYPE.variable),
            new Token(0x62, 0x9, "MaxX", SYNTAXTYPE.variable),
            new Token(0x62, 0xA, "MinY", SYNTAXTYPE.variable),
            new Token(0x62, 0xB, "MinY", SYNTAXTYPE.variable),
            new Token(0x62, 0xC, "ȳ", SYNTAXTYPE.variable),
            new Token(0x62, 0xD, "Σy", SYNTAXTYPE.variable),
            new Token(0x62, 0xE, "Σy²", SYNTAXTYPE.variable),
            new Token(0x62, 0xF, "Sy", SYNTAXTYPE.variable),
            new Token(0x62, 0x10, "σx", SYNTAXTYPE.variable),
            new Token(0x62, 0x11, "Σxy", SYNTAXTYPE.variable),
            new Token(0x62, 0x12, "ˢr", SYNTAXTYPE.variable),
            new Token(0x62, 0x13, "Med", SYNTAXTYPE.variable),
            new Token(0x62, 0x14, "Q₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x15, "Q₃", SYNTAXTYPE.variable),
            new Token(0x62, 0x16, "ˢa", SYNTAXTYPE.variable),
            new Token(0x62, 0x17, "ˢb", SYNTAXTYPE.variable),
            new Token(0x62, 0x18, "ˢc", SYNTAXTYPE.variable),
            new Token(0x62, 0x19, "ˢd", SYNTAXTYPE.variable),
            new Token(0x62, 0x1A, "ˢe", SYNTAXTYPE.variable),
            new Token(0x62, 0x1B, "x₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x1C, "x₂", SYNTAXTYPE.variable),
            new Token(0x62, 0x1D, "x₃", SYNTAXTYPE.variable),
            new Token(0x62, 0x1E, "y₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x1F, "y₂", SYNTAXTYPE.variable),
            new Token(0x62, 0x20, "y₃", SYNTAXTYPE.variable),
            new Token(0x62, 0x21, in, SYNTAXTYPE.variable),
            new Token(0x62, 0x22, "ˢp", SYNTAXTYPE.variable),
            new Token(0x62, 0x23, "ˢz", SYNTAXTYPE.variable),
            new Token(0x62, 0x24, "ˢt", SYNTAXTYPE.variable),
            new Token(0x62, 0x25, "χ²", SYNTAXTYPE.variable),
            new Token(0x62, 0x26, "Ϝ", SYNTAXTYPE.variable),
            new Token(0x62, 0x27, "df", SYNTAXTYPE.variable),
            new Token(0x62, 0x28, "p̂", SYNTAXTYPE.variable),
            new Token(0x62, 0x29, "p̂₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x2A, "p̂₂", SYNTAXTYPE.variable),
            new Token(0x62, 0x2B, "x̄₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x2C, "Sx₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x2D, "n₁", SYNTAXTYPE.variable),
            new Token(0x62, 0x2E, "x̄₂", SYNTAXTYPE.variable),
            new Token(0x62, 0x2F, "Sx₂", SYNTAXTYPE.variable),
            new Token(0x62, 0x30, "n₂", SYNTAXTYPE.variable),
            new Token(0x62, 0x31, "Sxp", SYNTAXTYPE.variable),
            new Token(0x62, 0x32, "lower", SYNTAXTYPE.variable),
            new Token(0x62, 0x33, "upper", SYNTAXTYPE.variable),
            new Token(0x62, 0x34, "ˢs", SYNTAXTYPE.variable),
            new Token(0x62, 0x35, "r²", SYNTAXTYPE.variable),
            new Token(0x62, 0x36, "R²", SYNTAXTYPE.variable),
            new Token(0x62, 0x37, "df", SYNTAXTYPE.variable),
            new Token(0x62, 0x38, "ˢ¹SS", SYNTAXTYPE.variable),
            new Token(0x62, 0x39, "ˢ¹MS", SYNTAXTYPE.variable),
            new Token(0x62, 0x3A, "df", SYNTAXTYPE.variable),
            new Token(0x62, 0x3B, "ˢ²SS", SYNTAXTYPE.variable),
            new Token(0x62, 0x3C, "ˢ²MS", SYNTAXTYPE.variable),
            new Token(0x63, 0x0, "ZXscl", SYNTAXTYPE.variable),
            new Token(0x63, 0x1, "ZYscl", SYNTAXTYPE.variable),
            new Token(0x63, 0x2, "Xscl", SYNTAXTYPE.variable),
            new Token(0x63, 0x3, "Yscl", SYNTAXTYPE.variable),
            new Token(0x63, 0x4, "u(" + in + "Min)", SYNTAXTYPE.variable),
            new Token(0x63, 0x5, "v(" + in + "Min)", SYNTAXTYPE.variable),
            new Token(0x63, 0x6, "U" + in + "⁻¹", SYNTAXTYPE.variable),
            new Token(0x63, 0x7, "V" + in + "⁻¹", SYNTAXTYPE.variable),
            new Token(0x63, 0x8, "ZU" + in + "Start", SYNTAXTYPE.variable),
            new Token(0x63, 0x9, "ZV" + in + "Start", SYNTAXTYPE.variable),
            new Token(0x63, 0xA, "Xmin", SYNTAXTYPE.variable),
            new Token(0x63, 0xB, "Xmax", SYNTAXTYPE.variable),
            new Token(0x63, 0xC, "Ymin", SYNTAXTYPE.variable),
            new Token(0x63, 0xD, "Ymax", SYNTAXTYPE.variable),
            new Token(0x63, 0xE, "Tmin", SYNTAXTYPE.variable),
            new Token(0x63, 0xF, "Tmax", SYNTAXTYPE.variable),
            new Token(0x63, 0x10, "θmin", SYNTAXTYPE.variable),
            new Token(0x63, 0x11, "θmax", SYNTAXTYPE.variable),
            new Token(0x63, 0x12, "ZXmin", SYNTAXTYPE.variable),
            new Token(0x63, 0x13, "ZXmax", SYNTAXTYPE.variable),
            new Token(0x63, 0x14, "ZYmin", SYNTAXTYPE.variable),
            new Token(0x63, 0x15, "ZYmax", SYNTAXTYPE.variable),
            new Token(0x63, 0x16, "Zθmin", SYNTAXTYPE.variable),
            new Token(0x63, 0x17, "Zθmax", SYNTAXTYPE.variable),
            new Token(0x63, 0x18, "ZTmin", SYNTAXTYPE.variable),
            new Token(0x63, 0x19, "ZTmax", SYNTAXTYPE.variable),
            new Token(0x63, 0x1A, "TblMin", SYNTAXTYPE.variable),
            new Token(0x63, 0x1B, in + "Min", SYNTAXTYPE.variable),
            new Token(0x63, 0x1C, "Z" + in + "Min", SYNTAXTYPE.variable),
            new Token(0x63, 0x1D, in + "Max", SYNTAXTYPE.variable),
            new Token(0x63, 0x1E, "Z" + in + "Max", SYNTAXTYPE.variable),
            new Token(0x63, 0x1F, in + "Start", SYNTAXTYPE.variable),
            new Token(0x63, 0x20, "Z" + in + "Start", SYNTAXTYPE.variable),
            new Token(0x63, 0x21, "△Tbl", SYNTAXTYPE.variable),
            new Token(0x63, 0x22, "TStep", SYNTAXTYPE.variable),
            new Token(0x63, 0x23, "θStep", SYNTAXTYPE.variable),
            new Token(0x63, 0x24, "ZTStep", SYNTAXTYPE.variable),
            new Token(0x63, 0x25, "ZθStep", SYNTAXTYPE.variable),
            new Token(0x63, 0x26, "△X", SYNTAXTYPE.variable),
            new Token(0x63, 0x27, "△Y", SYNTAXTYPE.variable),
            new Token(0x63, 0x28, "XFact", SYNTAXTYPE.variable),
            new Token(0x63, 0x29, "YFact", SYNTAXTYPE.variable),
            new Token(0x63, 0x2A, "TblInput", SYNTAXTYPE.variable),
            new Token(0x63, 0x2B, "ℕ", SYNTAXTYPE.variable),
            new Token(0x63, 0x2C, "I%", SYNTAXTYPE.variable),
            new Token(0x63, 0x2D, "PV", SYNTAXTYPE.variable),
            new Token(0x63, 0x2E, "PMT", SYNTAXTYPE.variable),
            new Token(0x63, 0x2F, "FV", SYNTAXTYPE.variable),
            new Token(0x63, 0x30, "Xres", SYNTAXTYPE.variable),
            new Token(0x63, 0x31, "ZXres", SYNTAXTYPE.variable), //267
            new Token(0x64, -1, "Radian", SYNTAXTYPE.setting),
            new Token(0x65, -1, "Degree", SYNTAXTYPE.setting),
            new Token(0x66, -1, "Normal", SYNTAXTYPE.setting),
            new Token(0x67, -1, "Sci", SYNTAXTYPE.setting),
            new Token(0x68, -1, "Eng", SYNTAXTYPE.setting),
            new Token(0x69, -1, "Float", SYNTAXTYPE.setting),
            new Token(0x6A, -1, "=", SYNTAXTYPE.operator),
            new Token(0x6B, -1, "<", SYNTAXTYPE.operator),
            new Token(0x6C, -1, ">", SYNTAXTYPE.operator),
            new Token(0x6D, -1, "≤", SYNTAXTYPE.operator),
            new Token(0x6E, -1, "≥", SYNTAXTYPE.operator),
            new Token(0x6F, -1, "≠", SYNTAXTYPE.operator),
            new Token(0x70, -1, "+", SYNTAXTYPE.operator),
            new Token(0x71, -1, "-", SYNTAXTYPE.operator),
            new Token(0x72, -1, "Ans", SYNTAXTYPE.variable),
            new Token(0x73, -1, "Fix ", SYNTAXTYPE.setting),
            new Token(0x74, -1, "Horiz", SYNTAXTYPE.setting),
            new Token(0x75, -1, "Full", SYNTAXTYPE.setting),
            new Token(0x76, -1, "Func", SYNTAXTYPE.setting),
            new Token(0x77, -1, "Param", SYNTAXTYPE.setting),
            new Token(0x78, -1, "Polar", SYNTAXTYPE.setting),
            new Token(0x79, -1, "Seq", SYNTAXTYPE.setting),
            new Token(0x7A, -1, "IndpntAuto", SYNTAXTYPE.setting),
            new Token(0x7B, -1, "IndpntAsk", SYNTAXTYPE.setting),
            new Token(0x7C, -1, "DependAuto", SYNTAXTYPE.setting),
            new Token(0x7D, -1, "DependAsk", SYNTAXTYPE.setting),
            new Token(0x7E, 0x0, "Sequential", SYNTAXTYPE.setting),
            new Token(0x7E, 0x1, "Simul", SYNTAXTYPE.setting),
            new Token(0x7E, 0x2, "PolarGC", SYNTAXTYPE.setting),
            new Token(0x7E, 0x3, "RectGC", SYNTAXTYPE.setting),
            new Token(0x7E, 0x4, "CoordOn", SYNTAXTYPE.setting),
            new Token(0x7E, 0x5, "CoordOff", SYNTAXTYPE.setting),
            new Token(0x7E, 0x6, "Connected", SYNTAXTYPE.setting),
            new Token(0x7E, 0x7, "Dot", SYNTAXTYPE.setting),
            new Token(0x7E, 0x8, "AxesOn", SYNTAXTYPE.setting),
            new Token(0x7E, 0x9, "AxesOff", SYNTAXTYPE.setting),
            new Token(0x7E, 0xA, "GridOn", SYNTAXTYPE.setting),
            new Token(0x7E, 0xB, "GridOff", SYNTAXTYPE.setting),
            new Token(0x7E, 0xC, "LabelOn", SYNTAXTYPE.setting),
            new Token(0x7E, 0xD, "LabelOff", SYNTAXTYPE.setting),
            new Token(0x7E, 0xE, "Web", SYNTAXTYPE.setting),
            new Token(0x7E, 0xF, "Time", SYNTAXTYPE.setting),
            new Token(0x7E, 0x10, "uvAxes", SYNTAXTYPE.setting),
            new Token(0x7E, 0x11, "vwAxes", SYNTAXTYPE.setting),
            new Token(0x7E, 0x13, "uwAxes", SYNTAXTYPE.setting),
            new Token(0x7F, -1, "▫", SYNTAXTYPE.operator),
            new Token(0x80, -1, "˖", SYNTAXTYPE.operator),
            new Token(0x81, -1, "·", SYNTAXTYPE.operator),
            new Token(0x82, -1, "*", SYNTAXTYPE.operator),
            new Token(0x83, -1, "/", SYNTAXTYPE.operator),
            new Token(0x84, -1, "Trace", SYNTAXTYPE.sub),
            new Token(0x85, -1, "ClrDraw", SYNTAXTYPE.sub),
            new Token(0x86, -1, "ZStandard", SYNTAXTYPE.sub),
            new Token(0x87, -1, "ZTrig", SYNTAXTYPE.sub),
            new Token(0x88, -1, "ZBox", SYNTAXTYPE.sub),
            new Token(0x89, -1, "Zoom In", SYNTAXTYPE.sub),
            new Token(0x8A, -1, "Zoom Out", SYNTAXTYPE.sub),
            new Token(0x8B, -1, "ZSquare", SYNTAXTYPE.sub),
            new Token(0x8C, -1, "ZInteger", SYNTAXTYPE.sub),
            new Token(0x8D, -1, "ZPrevious", SYNTAXTYPE.sub),
            new Token(0x8E, -1, "ZDecimal", SYNTAXTYPE.sub),
            new Token(0x8F, -1, "ZoomStat", SYNTAXTYPE.sub),
            new Token(0x90, -1, "ZoomRcl", SYNTAXTYPE.sub),
            new Token(0x91, -1, "PrintScreen", SYNTAXTYPE.nothing),
            new Token(0x92, -1, "ZoomSto", SYNTAXTYPE.sub),
            new Token(0x93, -1, "Text(", SYNTAXTYPE.function),
            new Token(0x94, -1, " nPr ", SYNTAXTYPE.operator),
            new Token(0x95, -1, " nCr ", SYNTAXTYPE.operator),
            new Token(0x96, -1, "FnOn ", SYNTAXTYPE.function),
            new Token(0x97, -1, "FnOff ", SYNTAXTYPE.function),
            new Token(0x98, -1, "StorePic ", SYNTAXTYPE.function),
            new Token(0x99, -1, "RecallPic ", SYNTAXTYPE.function),
            new Token(0x9A, -1, "StoreGDB", SYNTAXTYPE.function),
            new Token(0x9B, -1, "RecallGDB", SYNTAXTYPE.function),
            new Token(0x9C, -1, "Line(", SYNTAXTYPE.function),
            new Token(0x9D, -1, "Vertical ", SYNTAXTYPE.function),
            new Token(0x9E, -1, "Pt-On(", SYNTAXTYPE.function),
            new Token(0x9F, -1, "Pt-Off(", SYNTAXTYPE.function),
            new Token(0xA0, -1, "Pt-Change(", SYNTAXTYPE.function),
            new Token(0xA1, -1, "Pxl-On(", SYNTAXTYPE.function),
            new Token(0xA2, -1, "Pxl-Off(", SYNTAXTYPE.function),
            new Token(0xA3, -1, "Pxl-Change(", SYNTAXTYPE.function),
            new Token(0xA4, -1, "Shade(", SYNTAXTYPE.function),
            new Token(0xA5, -1, "Circle(", SYNTAXTYPE.function),
            new Token(0xA6, -1, "Horizontal ", SYNTAXTYPE.function),
            new Token(0xA7, -1, "Tangent(", SYNTAXTYPE.function),
            new Token(0xA8, -1, "DrawInv ", SYNTAXTYPE.function),
            new Token(0xA9, -1, "DrawF ", SYNTAXTYPE.function),
            new Token(0xAA, 0x0, "Str1", SYNTAXTYPE.variable),
            new Token(0xAA, 0x1, "Str2", SYNTAXTYPE.variable),
            new Token(0xAA, 0x2, "Str3", SYNTAXTYPE.variable),
            new Token(0xAA, 0x3, "Str4", SYNTAXTYPE.variable),
            new Token(0xAA, 0x4, "Str5", SYNTAXTYPE.variable),
            new Token(0xAA, 0x5, "Str6", SYNTAXTYPE.variable),
            new Token(0xAA, 0x6, "Str7", SYNTAXTYPE.variable),
            new Token(0xAA, 0x7, "Str8", SYNTAXTYPE.variable),
            new Token(0xAA, 0x8, "Str9", SYNTAXTYPE.variable),
            new Token(0xAA, 0x9, "Str10", SYNTAXTYPE.variable),
            new Token(0xAA, 0xA, "Str11", SYNTAXTYPE.variable),
            new Token(0xAA, 0xB, "Str12", SYNTAXTYPE.variable),
            new Token(0xAA, 0xC, "Str13", SYNTAXTYPE.variable),
            new Token(0xAA, 0xD, "Str14", SYNTAXTYPE.variable),
            new Token(0xAA, 0xE, "Str15", SYNTAXTYPE.variable),
            new Token(0xAA, 0xF, "Str16", SYNTAXTYPE.variable),
            new Token(0xAA, 0x10, "Str17", SYNTAXTYPE.variable),
            new Token(0xAA, 0x11, "Str18", SYNTAXTYPE.variable),
            new Token(0xAA, 0x12, "Str19", SYNTAXTYPE.variable),
            new Token(0xAA, 0x13, "Str20", SYNTAXTYPE.variable),
            new Token(0xAB, -1, "rand", SYNTAXTYPE.function),
            new Token(0xAC, -1, "π", SYNTAXTYPE.variable),
            new Token(0xAD, -1, "getKey", SYNTAXTYPE.function),
            new Token(0xAE, -1, "'", SYNTAXTYPE.operator),
            new Token(0xAF, -1, "?", SYNTAXTYPE.nothing),
            new Token(0xB0, -1, "­", SYNTAXTYPE.number),
            new Token(0xB1, -1, "int(", SYNTAXTYPE.function),
            new Token(0xB2, -1, "abs(", SYNTAXTYPE.function),
            new Token(0xB3, -1, "det(", SYNTAXTYPE.function),
            new Token(0xB4, -1, "identity(", SYNTAXTYPE.function),
            new Token(0xB5, -1, "dim(", SYNTAXTYPE.function),
            new Token(0xB6, -1, "sum(", SYNTAXTYPE.function),
            new Token(0xB7, -1, "prod(", SYNTAXTYPE.function),
            new Token(0xB8, -1, "not(", SYNTAXTYPE.function),
            new Token(0xB9, -1, "iPart(", SYNTAXTYPE.function),
            new Token(0xBA, -1, "fPart(", SYNTAXTYPE.function),
            new Token(0xBB, 0x0, "npv(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1, "irr(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2, "bal(", SYNTAXTYPE.function),
            new Token(0xBB, 0x3, "∑Prn(", SYNTAXTYPE.function),
            new Token(0xBB, 0x4, "∑Int(", SYNTAXTYPE.function),
            new Token(0xBB, 0x5, "►Nom(", SYNTAXTYPE.function),
            new Token(0xBB, 0x6, "►Eff(", SYNTAXTYPE.function),
            new Token(0xBB, 0x7, "dbd(", SYNTAXTYPE.function),
            new Token(0xBB, 0x8, "lcm(", SYNTAXTYPE.function),
            new Token(0xBB, 0x9, "gcd(", SYNTAXTYPE.function),
            new Token(0xBB, 0xA, "randInt(", SYNTAXTYPE.function),
            new Token(0xBB, 0xB, "randBin(", SYNTAXTYPE.function),
            new Token(0xBB, 0xC, "sub(", SYNTAXTYPE.function),
            new Token(0xBB, 0xD, "stdDev(", SYNTAXTYPE.function),
            new Token(0xBB, 0xE, "variance(", SYNTAXTYPE.function),
            new Token(0xBB, 0xF, "inString(", SYNTAXTYPE.function),
            new Token(0xBB, 0x10, "normalcdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x11, "invNorm(", SYNTAXTYPE.function),
            new Token(0xBB, 0x12, "tcdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x13, "χ²cdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x14, "Ϝcdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x15, "binompdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x16, "binomcdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x17, "poissonpdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x18, "poissoncdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x19, "geometpdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1A, "geometcdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1B, "normalpdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1C, "tpdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1D, "χ²pdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1E, "Ϝpdf(", SYNTAXTYPE.function),
            new Token(0xBB, 0x1F, "randNorm(", SYNTAXTYPE.function),
            new Token(0xBB, 0x20, "tvm_pmt", SYNTAXTYPE.function),
            new Token(0xBB, 0x21, "tmv_I%", SYNTAXTYPE.function),
            new Token(0xBB, 0x22, "tvm_PV", SYNTAXTYPE.function),
            new Token(0xBB, 0x23, "tvm_ℕ", SYNTAXTYPE.function),
            new Token(0xBB, 0x24, "tmv_FV", SYNTAXTYPE.function),
            new Token(0xBB, 0x25, "conj(", SYNTAXTYPE.function),
            new Token(0xBB, 0x26, "real(", SYNTAXTYPE.function),
            new Token(0xBB, 0x27, "imag(", SYNTAXTYPE.function),
            new Token(0xBB, 0x28, "angle(", SYNTAXTYPE.function),
            new Token(0xBB, 0x29, "cumSum(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2A, "expr(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2B, "legth(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2C, "ΔList(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2D, "ref(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2E, "rref(", SYNTAXTYPE.function),
            new Token(0xBB, 0x2F, "►Rect", SYNTAXTYPE.function),
            new Token(0xBB, 0x30, "►Polar", SYNTAXTYPE.function),
            new Token(0xBB, 0x31, "ℯ", SYNTAXTYPE.variable),
            new Token(0xBB, 0x32, "SinReg ", SYNTAXTYPE.function),
            new Token(0xBB, 0x33, "Logistic ", SYNTAXTYPE.function),
            new Token(0xBB, 0x34, "LinRegTTest ", SYNTAXTYPE.function),
            new Token(0xBB, 0x35, "ShadeNorm(", SYNTAXTYPE.function),
            new Token(0xBB, 0x36, "Shade_t(", SYNTAXTYPE.function),
            new Token(0xBB, 0x37, "Shadeχ²(", SYNTAXTYPE.function),
            new Token(0xBB, 0x38, "ShadeϜ(", SYNTAXTYPE.function),
            new Token(0xBB, 0x39, "Matr►list", SYNTAXTYPE.function),
            new Token(0xBB, 0x3A, "List►matr", SYNTAXTYPE.function),
            new Token(0xBB, 0x3B, "Z-Test(", SYNTAXTYPE.function),
            new Token(0xBB, 0x3C, "T-Test ", SYNTAXTYPE.function),
            new Token(0xBB, 0x3D, "2-SampZTest(", SYNTAXTYPE.function),
            new Token(0xBB, 0x3E, "1-PropZTest(", SYNTAXTYPE.function),
            new Token(0xBB, 0x3F, "2-PropZTest(", SYNTAXTYPE.function),
            new Token(0xBB, 0x40, "χ²-Test(", SYNTAXTYPE.function),
            new Token(0xBB, 0x41, "ZInterval ", SYNTAXTYPE.function),
            new Token(0xBB, 0x42, "2-SampZInt(", SYNTAXTYPE.function),
            new Token(0xBB, 0x43, "1-PropZInt(", SYNTAXTYPE.function),
            new Token(0xBB, 0x44, "2-PropZInt(", SYNTAXTYPE.function),
            new Token(0xBB, 0x45, "GraphStyle(", SYNTAXTYPE.function),
            new Token(0xBB, 0x46, "2-SampTTest ", SYNTAXTYPE.function),
            new Token(0xBB, 0x47, "2-SampϜTest ", SYNTAXTYPE.function),
            new Token(0xBB, 0x48, "TInterval ", SYNTAXTYPE.function),
            new Token(0xBB, 0x49, "2-SampTInt ", SYNTAXTYPE.function),
            new Token(0xBB, 0x4A, "SetUpEditor ", SYNTAXTYPE.function),
            new Token(0xBB, 0x4B, "PMT_End", SYNTAXTYPE.function),
            new Token(0xBB, 0x4C, "PMT_Bgn", SYNTAXTYPE.function),
            new Token(0xBB, 0x4D, "Real", SYNTAXTYPE.setting),
            new Token(0xBB, 0x4E, "rℯ^θ" + ii, SYNTAXTYPE.setting),
            new Token(0xBB, 0x4F, "a+b" + ii, SYNTAXTYPE.setting),
            new Token(0xBB, 0x50, "ExprOn", SYNTAXTYPE.function),
            new Token(0xBB, 0x51, "ExprOff", SYNTAXTYPE.function),
            new Token(0xBB, 0x52, "ClrAllLists", SYNTAXTYPE.sub),
            new Token(0xBB, 0x53, "GetCalc(", SYNTAXTYPE.function),
            new Token(0xBB, 0x54, "DelVar ", SYNTAXTYPE.function),
            new Token(0xBB, 0x55, "Equ►String(", SYNTAXTYPE.function),
            new Token(0xBB, 0x56, "String►Equ(", SYNTAXTYPE.function),
            new Token(0xBB, 0x57, "Clear Entries", SYNTAXTYPE.sub),
            new Token(0xBB, 0x58, "Select(", SYNTAXTYPE.function),
            new Token(0xBB, 0x59, "ANOVA(", SYNTAXTYPE.function),
            new Token(0xBB, 0x5A, "ModBoxPlot", SYNTAXTYPE.setting),
            new Token(0xBB, 0x5B, "NormProbPlot", SYNTAXTYPE.setting),
            new Token(0xBB, 0x64, "G-T", SYNTAXTYPE.setting),
            new Token(0xBB, 0x65, "ZoomFit", SYNTAXTYPE.setting),
            new Token(0xBB, 0x66, "DiagnosticOn", SYNTAXTYPE.setting),
            new Token(0xBB, 0x67, "DiagnosticOff", SYNTAXTYPE.setting),
            new Token(0xBB, 0x68, "Archive ", SYNTAXTYPE.function),
            new Token(0xBB, 0x69, "UnArchive ", SYNTAXTYPE.function),
            new Token(0xBB, 0x6A, "Asm(", SYNTAXTYPE.function),
            new Token(0xBB, 0x6B, "AsmComp(", SYNTAXTYPE.function),
            new Token(0xBB, 0x6C, "AsmPrgm", SYNTAXTYPE.function),
            new Token(0xBB, 0x6E, "Á", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x6F, "À", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x70, "Â", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x71, "Ä", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x72, "á", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x73, "à", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x74, "â", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x75, "ä", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x76, "É", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x77, "È", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x78, "Ê", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x79, "Ë", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x7A, "é", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x7B, "è", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x7C, "ê", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x7D, "ë", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x7F, "Ì", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x80, "Î", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x81, "Ï", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x82, "í", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x83, "ì", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x84, "î", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x85, "ï", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x86, "Ó", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x87, "Ò", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x88, "Ô", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x89, "Ö", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x8A, "ó", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x8B, "ò", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x8C, "ô", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x8D, "ö", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x8E, "Ú", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x8F, "Ù", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x90, "Û", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x91, "Ü", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x92, "ú", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x93, "ù", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x94, "û", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x95, "ú", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x96, "Ç", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x97, "ç", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x98, "Ñ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x99, "ñ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x9A, "ˊ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x9B, "ˋ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x9C, "¨", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x9D, "¿", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x9E, "¡", SYNTAXTYPE.nothing),
            new Token(0xBB, 0x9F, "α", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA0, "β", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA1, "γ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA2, "Δ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA3, "δ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA4, "ε", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA5, "λ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA6, "μ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA7, "π", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA8, "ρ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xA9, "∑", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xAB, "Φ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xAC, "Ω", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xAD, "p̂", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xAE, "χ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xAF, "Ϝ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB0, "a", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB1, "b", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB2, "c", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB3, "d", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB4, "e", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB5, "f", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB6, "g", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB7, "h", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB8, "i", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xB9, "j", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xBA, "k", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xBC, "l", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xBD, "m", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xBE, "n", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xBF, "o", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC0, "p", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC1, "q", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC2, "r", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC3, "s", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC4, "t", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC5, "u", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC6, "v", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC7, "w", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC8, "x", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xC9, "y", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xCA, "z", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xCB, "σ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xCC, "τ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xCD, "Í", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xCE, "GarbageCollect", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xCF, "~", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD0, "'", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD1, "@", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD2, "#", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD3, "$", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD4, "&", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD5, "`", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD6, ";", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD7, "\\", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD8, "|", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xD9, "_", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xDA, "%", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xDB, "…", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xDC, "∠", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xDD, "Β", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xDE, "ˣ", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE0, "⊤", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE1, "₀", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE2, "₁", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE3, "₂", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE4, "₃", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE5, "₄", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE6, "₅", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE7, "₆", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE8, "₇", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xE9, "₈", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xEA, "₉", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xEB, "₁₀", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xEC, "◄", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xED, "►", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xEE, "↑", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xF0, "↓", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xF2, "⨯", SYNTAXTYPE.nothing), //x
            new Token(0xBB, 0xF3, "∫", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xF4, "⬆", SYNTAXTYPE.nothing), //Up
            new Token(0xBB, 0xF5, "⬇", SYNTAXTYPE.nothing),//Down
            new Token(0xBB, 0xF6, "√", SYNTAXTYPE.nothing),
            new Token(0xBB, 0xF7, "⌸", SYNTAXTYPE.nothing), //Inverse equals (Box)
            new Token(0xBC, -1, "√(", SYNTAXTYPE.function),
            new Token(0xBD, -1, "∛(", SYNTAXTYPE.function),
            new Token(0xBE, -1, "ln(", SYNTAXTYPE.function),
            new Token(0xBF, -1, "ℯ^(", SYNTAXTYPE.function),
            new Token(0xC0, -1, "log(", SYNTAXTYPE.function),
            new Token(0xC1, -1, "₁₀^(", SYNTAXTYPE.function),
            new Token(0xC2, -1, "sin(", SYNTAXTYPE.function),
            new Token(0xC3, -1, "sin⁻¹(", SYNTAXTYPE.function),
            new Token(0xC4, -1, "cos(", SYNTAXTYPE.function),
            new Token(0xC5, -1, "cos⁻¹(", SYNTAXTYPE.function),
            new Token(0xC6, -1, "tan(", SYNTAXTYPE.function),
            new Token(0xC7, -1, "tan⁻¹(", SYNTAXTYPE.function),
            new Token(0xC8, -1, "sinh(", SYNTAXTYPE.function),
            new Token(0xC9, -1, "sinh⁻¹(", SYNTAXTYPE.function),
            new Token(0xCA, -1, "cosh(", SYNTAXTYPE.function),
            new Token(0xCB, -1, "cosh⁻¹(", SYNTAXTYPE.function),
            new Token(0xCC, -1, "tanh(", SYNTAXTYPE.function),
            new Token(0xCD, -1, "tanh⁻¹(", SYNTAXTYPE.function),
            new Token(0xCE, -1, "If ", SYNTAXTYPE.keyword),
            new Token(0xCF, -1, "Then", SYNTAXTYPE.keyword),
            new Token(0xD0, -1, "Else", SYNTAXTYPE.keyword),
            new Token(0xD1, -1, "While ", SYNTAXTYPE.keyword),
            new Token(0xD2, -1, "Repeat ", SYNTAXTYPE.keyword),
            new Token(0xD3, -1, "For(", SYNTAXTYPE.keyword),
            new Token(0xD4, -1, "End", SYNTAXTYPE.keyword),
            new Token(0xD5, -1, "Return", SYNTAXTYPE.keyword),
            new Token(0xD6, -1, "Lbl ", SYNTAXTYPE.keyword),
            new Token(0xD7, -1, "Goto ", SYNTAXTYPE.keyword),
            new Token(0xD8, -1, "Pause ", SYNTAXTYPE.keyword),
            new Token(0xD9, -1, "Stop", SYNTAXTYPE.keyword),
            new Token(0xDA, -1, "IS>(", SYNTAXTYPE.function),
            new Token(0xDB, -1, "DS>(", SYNTAXTYPE.function),
            new Token(0xDC, -1, "Input ", SYNTAXTYPE.function),
            new Token(0xDD, -1, "Prompt ", SYNTAXTYPE.function),
            new Token(0xDE, -1, "Disp ", SYNTAXTYPE.function),
            new Token(0xDF, -1, "DispGraph", SYNTAXTYPE.function),
            new Token(0xE0, -1, "Output(", SYNTAXTYPE.function),
            new Token(0xE1, -1, "ClrHome", SYNTAXTYPE.function),
            new Token(0xE2, -1, "Fill(", SYNTAXTYPE.function),
            new Token(0xE3, -1, "SortA(", SYNTAXTYPE.function),
            new Token(0xE4, -1, "SortD(", SYNTAXTYPE.function),
            new Token(0xE5, -1, "DispTable", SYNTAXTYPE.sub),
            new Token(0xE6, -1, "Menu(", SYNTAXTYPE.function),
            new Token(0xE7, -1, "Send(", SYNTAXTYPE.function),
            new Token(0xE8, -1, "Get(", SYNTAXTYPE.function),
            new Token(0xE9, -1, "PlotsOn ", SYNTAXTYPE.function),
            new Token(0xEA, -1, "PlotsOff ", SYNTAXTYPE.function),
            new Token(0xEB, -1, "˪", SYNTAXTYPE.variable), //L
            new Token(0xEC, -1, "Plot1(", SYNTAXTYPE.function),
            new Token(0xED, -1, "Plot2(", SYNTAXTYPE.function),
            new Token(0xEE, -1, "Plot3(", SYNTAXTYPE.function),
            new Token(0xEF, 0x0, "setDate(", SYNTAXTYPE.function),
            new Token(0xEF, 0x1, "setTime(", SYNTAXTYPE.function),
            new Token(0xEF, 0x2, "checkTmr(", SYNTAXTYPE.function),
            new Token(0xEF, 0x3, "setDtFmt(", SYNTAXTYPE.function),
            new Token(0xEF, 0x4, "setTmFmt(", SYNTAXTYPE.function),
            new Token(0xEF, 0x5, "TimeCnv(", SYNTAXTYPE.function),
            new Token(0xEF, 0x6, "dayOfWk(", SYNTAXTYPE.function),
            new Token(0xEF, 0x7, "getDtStr(", SYNTAXTYPE.function),
            new Token(0xEF, 0x8, "getTmStr(", SYNTAXTYPE.function),
            new Token(0xEF, 0x9, "getDate", SYNTAXTYPE.function),
            new Token(0xEF, 0xA, "getTime", SYNTAXTYPE.function),
            new Token(0xEF, 0xB, "startTmr", SYNTAXTYPE.function),
            new Token(0xEF, 0xC, "getDtFmt", SYNTAXTYPE.function),
            new Token(0xEF, 0xD, "getTmFmt", SYNTAXTYPE.function),
            new Token(0xEF, 0xE, "isClockOn", SYNTAXTYPE.function),
            new Token(0xEF, 0xF, "ClockOff", SYNTAXTYPE.setting),
            new Token(0xEF, 0x10, "ClockOn", SYNTAXTYPE.setting),
            new Token(0xEF, 0x11, "OpenLib(", SYNTAXTYPE.function),
            new Token(0xEF, 0x12, "ExecLib", SYNTAXTYPE.function),
            new Token(0xF0, -1, "^", SYNTAXTYPE.operator),
            new Token(0xF1, -1, "ˣ√", SYNTAXTYPE.operator),
            new Token(0xF2, -1, "1-Var Stats ", SYNTAXTYPE.function),
            new Token(0xF3, -1, "2-Var Stats ", SYNTAXTYPE.function),
            new Token(0xF4, -1, "LinReg(a+bx) ", SYNTAXTYPE.function),
            new Token(0xF5, -1, "ExpReg ", SYNTAXTYPE.function),
            new Token(0xF6, -1, "LnReg ", SYNTAXTYPE.function),
            new Token(0xF7, -1, "PwrReg ", SYNTAXTYPE.function),
            new Token(0xF8, -1, "Med-Med ", SYNTAXTYPE.function),
            new Token(0xF9, -1, "QuadReg ", SYNTAXTYPE.function),
            new Token(0xFA, -1, "ClrList ", SYNTAXTYPE.function),
            new Token(0xFB, -1, "ClrTable", SYNTAXTYPE.sub),
            new Token(0xFC, -1, "Histogram", SYNTAXTYPE.setting),
            new Token(0xFD, -1, "xyLine", SYNTAXTYPE.setting),
            new Token(0xFE, -1, "Scatter", SYNTAXTYPE.setting),
            new Token(0xFF, -1, "LinReg(ax+b) ", SYNTAXTYPE.setting)
        };
    }
}
